@@ -7,7 +7,6 @@ use std::convert::TryFrom;
 use std::convert::TryInto;
 use std::fmt::Debug;
 use std::hash::{Hash, Hasher};
-use std::marker::PhantomData;
 use std::ops::Index;
 use std::ops::IndexMut;
 use std::string::String;
@@ -26,12 +25,14 @@ impl<T> Embeddable for T where T: Domain + Clone {}
 
 impl<D: Domain> Domain for Arc<D> {}
 
-pub trait NestedValue<D: Embeddable>: Sized + Debug + Clone + Eq + Hash + Ord {
-    fn new<V>(v: V) -> Self where Value<Self, D>: From<V> {
+pub trait NestedValue: Sized + Debug + Clone + Eq + Hash + Ord {
+    type Embedded: Embeddable;
+
+    fn new<V>(v: V) -> Self where Value<Self>: From<V> {
         Value::from(v).wrap()
     }
 
-    fn domain<E>(e: E) -> Self where D: From<E> {
+    fn domain<E>(e: E) -> Self where Self::Embedded: From<E> {
         Value::Embedded(e.into()).wrap()
     }
 
@@ -43,12 +44,12 @@ pub trait NestedValue<D: Embeddable>: Sized + Debug + Clone + Eq + Hash + Ord {
         Value::bytestring(v).wrap()
     }
 
-    fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self;
+    fn wrap(anns: Annotations<Self>, v: Value<Self>) -> Self;
 
-    fn annotations(&self) -> &Annotations<Self, D>;
-    fn value(&self) -> &Value<Self, D>;
-    fn pieces(self) -> (Annotations<Self, D>, Value<Self, D>);
-    fn value_owned(self) -> Value<Self, D>;
+    fn annotations(&self) -> &Annotations<Self>;
+    fn value(&self) -> &Value<Self>;
+    fn pieces(self) -> (Annotations<Self>, Value<Self>);
+    fn value_owned(self) -> Value<Self>;
 
     fn value_class(&self) -> ValueClass {
         self.value().value_class()
@@ -61,20 +62,24 @@ pub trait NestedValue<D: Embeddable>: Sized + Debug + Clone + Eq + Hash + Ord {
         self.value().fmt(f)
     }
 
-    fn copy_via<M: NestedValue<E>, E: Embeddable, F, Err>(&self, f: &mut F) -> Result<M, Err>
+    fn strip_annotations<M: NestedValue<Embedded = Self::Embedded>>(&self) -> M {
+        M::wrap(Annotations::empty(), self.value().strip_annotations())
+    }
+
+    fn copy_via<M: NestedValue, F, Err>(&self, f: &mut F) -> Result<M, Err>
     where
-        F: FnMut(&D) -> Result<Value<M, E>, Err>
+        F: FnMut(&Self::Embedded) -> Result<Value<M>, Err>
     {
         Ok(M::wrap(self.annotations().copy_via(f)?, self.value().copy_via(f)?))
     }
 
     fn foreach_embedded<F, Err>(&self, f: &mut F) -> Result<(), Err>
     where
-        F: FnMut(&D) -> Result<(), Err>
+        F: FnMut(&Self::Embedded) -> Result<(), Err>
     {
         match &self.annotations().0 {
             None => (),
-            Some(vs) => for v in vs { v.foreach_embedded(f)? },
+            Some(vs) => for v in vs.iter() { v.foreach_embedded(f)? },
         }
         self.value().foreach_embedded(f)
     }
@@ -82,7 +87,7 @@ pub trait NestedValue<D: Embeddable>: Sized + Debug + Clone + Eq + Hash + Ord {
 
 /// The `Value`s from the specification.
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub enum Value<N, D> where N: NestedValue<D>, D: Embeddable {
+pub enum Value<N: NestedValue> {
     Boolean(bool),
     Float(Float),
     Double(Double),
@@ -94,7 +99,7 @@ pub enum Value<N, D> where N: NestedValue<D>, D: Embeddable {
     Sequence(Vec<N>),
     Set(Set<N>),
     Dictionary(Map<N, N>),
-    Embedded(D),
+    Embedded(N::Embedded),
 }
 
 /// The kinds of `Value` from the specification.
@@ -167,7 +172,7 @@ impl<N> Record<N> {
         &mut self.0
     }
 
-    pub fn finish<D: Embeddable>(self) -> Value<N, D> where N: NestedValue<D> {
+    pub fn finish(self) -> Value<N> where N: NestedValue {
         Value::Record(self)
     }
 }
@@ -256,41 +261,44 @@ impl PartialOrd for Double {
 
 impl Eq for Double {}
 
-impl<N: NestedValue<D>, D: Embeddable> From<bool> for Value<N, D> { fn from(v: bool) -> Self { Value::Boolean(v) } }
+impl<N: NestedValue> From<bool> for Value<N> { fn from(v: bool) -> Self { Value::Boolean(v) } }
+impl<N: NestedValue> From<&bool> for Value<N> { fn from(v: &bool) -> Self { Value::Boolean(*v) } }
 
-impl<N: NestedValue<D>, D: Embeddable> From<f32> for Value<N, D> { fn from(v: f32) -> Self { Value::Float(Float::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<&Float> for Value<N, D> { fn from(v: &Float) -> Self { Value::Float(v.clone()) } }
-impl<N: NestedValue<D>, D: Embeddable> From<f64> for Value<N, D> { fn from(v: f64) -> Self { Value::Double(Double::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<&Double> for Value<N, D> { fn from(v: &Double) -> Self { Value::Double(v.clone()) } }
+impl<N: NestedValue> From<f32> for Value<N> { fn from(v: f32) -> Self { Value::Float(Float::from(v)) } }
+impl<N: NestedValue> From<&f32> for Value<N> { fn from(v: &f32) -> Self { Value::Float(Float::from(*v)) } }
+impl<N: NestedValue> From<&Float> for Value<N> { fn from(v: &Float) -> Self { Value::Float(v.clone()) } }
+impl<N: NestedValue> From<f64> for Value<N> { fn from(v: f64) -> Self { Value::Double(Double::from(v)) } }
+impl<N: NestedValue> From<&f64> for Value<N> { fn from(v: &f64) -> Self { Value::Double(Double::from(*v)) } }
+impl<N: NestedValue> From<&Double> for Value<N> { fn from(v: &Double) -> Self { Value::Double(v.clone()) } }
 
-impl<N: NestedValue<D>, D: Embeddable> From<u8> for Value<N, D> { fn from(v: u8) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<i8> for Value<N, D> { fn from(v: i8) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<u16> for Value<N, D> { fn from(v: u16) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<i16> for Value<N, D> { fn from(v: i16) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<u32> for Value<N, D> { fn from(v: u32) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<i32> for Value<N, D> { fn from(v: i32) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<u64> for Value<N, D> { fn from(v: u64) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<i64> for Value<N, D> { fn from(v: i64) -> Self { Value::from(i128::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<usize> for Value<N, D> { fn from(v: usize) -> Self { Value::from(v as u128) } }
-impl<N: NestedValue<D>, D: Embeddable> From<isize> for Value<N, D> { fn from(v: isize) -> Self { Value::from(v as i128) } }
-impl<N: NestedValue<D>, D: Embeddable> From<u128> for Value<N, D> { fn from(v: u128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<i128> for Value<N, D> { fn from(v: i128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<&BigInt> for Value<N, D> { fn from(v: &BigInt) -> Self { Value::SignedInteger(SignedInteger::from(Cow::Borrowed(v))) } }
-impl<N: NestedValue<D>, D: Embeddable> From<BigInt> for Value<N, D> { fn from(v: BigInt) -> Self { Value::SignedInteger(SignedInteger::from(Cow::Owned(v))) } }
-impl<N: NestedValue<D>, D: Embeddable> From<&SignedInteger> for Value<N, D> { fn from(v: &SignedInteger) -> Self { Value::SignedInteger(v.clone()) } }
+impl<N: NestedValue> From<u8> for Value<N> { fn from(v: u8) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<i8> for Value<N> { fn from(v: i8) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<u16> for Value<N> { fn from(v: u16) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<i16> for Value<N> { fn from(v: i16) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<u32> for Value<N> { fn from(v: u32) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<i32> for Value<N> { fn from(v: i32) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<u64> for Value<N> { fn from(v: u64) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<i64> for Value<N> { fn from(v: i64) -> Self { Value::from(i128::from(v)) } }
+impl<N: NestedValue> From<usize> for Value<N> { fn from(v: usize) -> Self { Value::from(v as u128) } }
+impl<N: NestedValue> From<isize> for Value<N> { fn from(v: isize) -> Self { Value::from(v as i128) } }
+impl<N: NestedValue> From<u128> for Value<N> { fn from(v: u128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
+impl<N: NestedValue> From<i128> for Value<N> { fn from(v: i128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
+impl<N: NestedValue> From<&BigInt> for Value<N> { fn from(v: &BigInt) -> Self { Value::SignedInteger(SignedInteger::from(Cow::Borrowed(v))) } }
+impl<N: NestedValue> From<BigInt> for Value<N> { fn from(v: BigInt) -> Self { Value::SignedInteger(SignedInteger::from(Cow::Owned(v))) } }
+impl<N: NestedValue> From<&SignedInteger> for Value<N> { fn from(v: &SignedInteger) -> Self { Value::SignedInteger(v.clone()) } }
 
-impl<N: NestedValue<D>, D: Embeddable> From<&str> for Value<N, D> { fn from(v: &str) -> Self { Value::String(String::from(v)) } }
-impl<N: NestedValue<D>, D: Embeddable> From<String> for Value<N, D> { fn from(v: String) -> Self { Value::String(v) } }
-impl<N: NestedValue<D>, D: Embeddable> From<&String> for Value<N, D> { fn from(v: &String) -> Self { Value::String(v.to_owned()) } }
+impl<N: NestedValue> From<&str> for Value<N> { fn from(v: &str) -> Self { Value::String(String::from(v)) } }
+impl<N: NestedValue> From<String> for Value<N> { fn from(v: String) -> Self { Value::String(v) } }
+impl<N: NestedValue> From<&String> for Value<N> { fn from(v: &String) -> Self { Value::String(v.to_owned()) } }
 
-impl<N: NestedValue<D>, D: Embeddable> From<&[u8]> for Value<N, D> { fn from(v: &[u8]) -> Self { Value::ByteString(Vec::from(v)) } }
-// impl<N: NestedValue<D>, D: Embeddable> From<Vec<u8>> for Value<N, D> { fn from(v: Vec<u8>) -> Self { Value::ByteString(v) } }
+impl<N: NestedValue> From<&[u8]> for Value<N> { fn from(v: &[u8]) -> Self { Value::ByteString(Vec::from(v)) } }
+// impl<N: NestedValue> From<Vec<u8>> for Value<N> { fn from(v: Vec<u8>) -> Self { Value::ByteString(v) } }
 
-impl<N: NestedValue<D>, D: Embeddable> From<Vec<N>> for Value<N, D> { fn from(v: Vec<N>) -> Self { Value::Sequence(v) } }
-impl<N: NestedValue<D>, D: Embeddable> From<Set<N>> for Value<N, D> { fn from(v: Set<N>) -> Self { Value::Set(v) } }
-impl<N: NestedValue<D>, D: Embeddable> From<Map<N, N>> for Value<N, D> { fn from(v: Map<N, N>) -> Self { Value::Dictionary(v) } }
+impl<N: NestedValue> From<Vec<N>> for Value<N> { fn from(v: Vec<N>) -> Self { Value::Sequence(v) } }
+impl<N: NestedValue> From<Set<N>> for Value<N> { fn from(v: Set<N>) -> Self { Value::Set(v) } }
+impl<N: NestedValue> From<Map<N, N>> for Value<N> { fn from(v: Map<N, N>) -> Self { Value::Dictionary(v) } }
 
-impl<N: NestedValue<D>, D: Embeddable> Debug for Value<N, D> {
+impl<N: NestedValue> Debug for Value<N> {
     // Not *quite* a formatter for the Preserves text syntax, since it
     // doesn't escape strings/symbols properly.
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -336,7 +344,7 @@ impl<N: NestedValue<D>, D: Embeddable> Debug for Value<N, D> {
 
 //---------------------------------------------------------------------------
 
-impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
+impl<N: NestedValue> Value<N> {
     pub fn wrap(self) -> N {
         N::wrap(Annotations::empty(), self)
     }
@@ -687,7 +695,7 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         self.as_bytestring().ok_or_else(|| self.expected(ExpectedKind::ByteString))
     }
 
-    pub fn symbol(s: &str) -> Value<N, D> {
+    pub fn symbol(s: &str) -> Value<N> {
         Value::Symbol(s.to_string())
     }
 
@@ -737,6 +745,13 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         }
     }
 
+    pub fn into_record(self) -> Option<Record<N>> {
+        match self {
+            Value::Record(r) => Some(r),
+            _ => None,
+        }
+    }
+
     pub fn as_record_mut(&mut self, arity: Option<usize>) -> Option<&mut Record<N>> {
         if let Value::Record(r) = self {
             match arity {
@@ -757,11 +772,11 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         Self::record(N::symbol(label), expected_arity)
     }
 
-    pub fn simple_record0(label: &str) -> Value<N, D> {
+    pub fn simple_record0(label: &str) -> Value<N> {
         Self::simple_record(label, 0).finish()
     }
 
-    pub fn simple_record1(label: &str, field: N) -> Value<N, D> {
+    pub fn simple_record1(label: &str, field: N) -> Value<N> {
         let mut r = Self::simple_record(label, 1);
         r.fields_vec_mut().push(field);
         r.finish()
@@ -809,6 +824,13 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         }
     }
 
+    pub fn into_sequence(self) -> Option<Vec<N>> {
+        match self {
+            Value::Sequence(s) => Some(s),
+            _ => None,
+        }
+    }
+
     pub fn as_sequence_mut(&mut self) -> Option<&mut Vec<N>> {
         if let Value::Sequence(s) = self {
             Some(s)
@@ -833,6 +855,13 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         }
     }
 
+    pub fn into_set(self) -> Option<Set<N>> {
+        match self {
+            Value::Set(s) => Some(s),
+            _ => None,
+        }
+    }
+
     pub fn as_set_mut(&mut self) -> Option<&mut Set<N>> {
         if let Value::Set(s) = self {
             Some(s)
@@ -857,6 +886,13 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         }
     }
 
+    pub fn into_dictionary(self) -> Option<Map<N, N>> {
+        match self {
+            Value::Dictionary(s) => Some(s),
+            _ => None,
+        }
+    }
+
     pub fn as_dictionary_mut(&mut self) -> Option<&mut Map<N, N>> {
         if let Value::Dictionary(s) = self {
             Some(s)
@@ -873,7 +909,7 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         self.as_embedded().is_some()
     }
 
-    pub fn as_embedded(&self) -> Option<&D> {
+    pub fn as_embedded(&self) -> Option<&N::Embedded> {
         if let Value::Embedded(d) = self {
             Some(d)
         } else {
@@ -881,13 +917,34 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         }
     }
 
-    pub fn to_embedded(&self) -> Result<&D, Error> {
+    pub fn to_embedded(&self) -> Result<&N::Embedded, Error> {
         self.as_embedded().ok_or_else(|| self.expected(ExpectedKind::Embedded))
     }
 
-    pub fn copy_via<M: NestedValue<E>, E: Embeddable, F, Err>(&self, f: &mut F) -> Result<Value<M, E>, Err>
+    pub fn strip_annotations<M: NestedValue<Embedded = N::Embedded>>(&self) -> Value<M> {
+        match self {
+            Value::Boolean(b) => Value::Boolean(*b),
+            Value::Float(f) => Value::Float(f.clone()),
+            Value::Double(d) => Value::Double(d.clone()),
+            Value::SignedInteger(n) => Value::SignedInteger(n.clone()),
+            Value::String(s) => Value::String(s.clone()),
+            Value::ByteString(v) => Value::ByteString(v.clone()),
+            Value::Symbol(v) => Value::Symbol(v.clone()),
+            Value::Record(r) => Value::Record(Record(r.fields_vec().iter().map(
+                |a| a.strip_annotations()).collect())),
+            Value::Sequence(v) => Value::Sequence(v.iter().map(
+                |a| a.strip_annotations()).collect()),
+            Value::Set(v) => Value::Set(v.iter().map(
+                |a| a.strip_annotations()).collect()),
+            Value::Dictionary(v) => Value::Dictionary(v.iter().map(
+                |(a,b)| (a.strip_annotations(), b.strip_annotations())).collect()),
+            Value::Embedded(d) => Value::Embedded(d.clone()),
+        }
+    }
+
+    pub fn copy_via<M: NestedValue, F, Err>(&self, f: &mut F) -> Result<Value<M>, Err>
     where
-        F: FnMut(&D) -> Result<Value<M, E>, Err>
+        F: FnMut(&N::Embedded) -> Result<Value<M>, Err>
     {
         Ok(match self {
             Value::Boolean(b) => Value::Boolean(*b),
@@ -913,7 +970,7 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
 
     pub fn foreach_embedded<F, Err>(&self, f: &mut F) -> Result<(), Err>
     where
-        F: FnMut(&D) -> Result<(), Err>
+        F: FnMut(&N::Embedded) -> Result<(), Err>
     {
         match self {
             Value::Boolean(_) |
@@ -935,7 +992,7 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> Index<usize> for Value<N, D> {
+impl<N: NestedValue> Index<usize> for Value<N> {
     type Output = N;
 
     fn index(&self, i: usize) -> &Self::Output {
@@ -943,13 +1000,13 @@ impl<N: NestedValue<D>, D: Embeddable> Index<usize> for Value<N, D> {
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> IndexMut<usize> for Value<N, D> {
+impl<N: NestedValue> IndexMut<usize> for Value<N> {
     fn index_mut(&mut self, i: usize) -> &mut Self::Output {
         &mut self.as_sequence_mut().unwrap()[i]
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> Index<&N> for Value<N, D> {
+impl<N: NestedValue> Index<&N> for Value<N> {
     type Output = N;
 
     fn index(&self, i: &N) -> &Self::Output {
@@ -975,16 +1032,15 @@ impl<'de> serde::Deserialize<'de> for UnwrappedIOValue {
 //---------------------------------------------------------------------------
 
 #[derive(Clone)]
-pub struct Annotations<N: NestedValue<D>, D: Embeddable>(Option<Vec<N>>, PhantomData<D>);
+pub struct Annotations<N: NestedValue>(Option<Box<Vec<N>>>);
 
-impl<N: NestedValue<D>, D: Embeddable> Annotations<N, D> {
+impl<N: NestedValue> Annotations<N> {
     pub fn empty() -> Self {
-        Annotations(None,
-                    PhantomData)
+        Annotations(None)
     }
 
     pub fn new(anns: Option<Vec<N>>) -> Self {
-        Annotations(anns, PhantomData)
+        Annotations(anns.map(Box::new))
     }
 
     pub fn maybe_slice(&self) -> Option<&[N]> {
@@ -998,26 +1054,43 @@ impl<N: NestedValue<D>, D: Embeddable> Annotations<N, D> {
         self.maybe_slice().unwrap_or(&[])
     }
 
+    pub fn to_vec(self) -> Vec<N> {
+        use std::ops::DerefMut;
+        self.0.map(|mut b| std::mem::take(b.deref_mut())).unwrap_or_default()
+    }
+
     pub fn modify<F>(&mut self, f: F) -> &mut Self
     where
         F: FnOnce(&mut Vec<N>)
     {
-        let mut v: Vec<N> = self.0.take().unwrap_or_else(Vec::new);
-        f(&mut v);
-        self.0 = if v.is_empty() { None } else { Some(v) };
+        match &mut self.0 {
+            None => {
+                let mut v = Vec::new();
+                f(&mut v);
+                if !v.is_empty() {
+                    self.0 = Some(Box::new(v));
+                }
+            }
+            Some(b) => {
+                use std::ops::DerefMut;
+                f(b.deref_mut());
+                if b.is_empty() {
+                    self.0 = None;
+                }
+            }
+        }
         self
     }
 
-    pub fn copy_via<M: NestedValue<E>, E: Embeddable, F, Err>(&self, f: &mut F) -> Result<Annotations<M, E>, Err>
+    pub fn copy_via<M: NestedValue, F, Err>(&self, f: &mut F) -> Result<Annotations<M>, Err>
     where
-        F: FnMut(&D) -> Result<Value<M, E>, Err>
+        F: FnMut(&N::Embedded) -> Result<Value<M>, Err>
     {
         Ok(match &self.0 {
             None =>
-                Annotations(None, PhantomData),
+                Annotations(None),
             Some(b) =>
-                Annotations(Some(b.iter().map(|a| a.copy_via(f)).collect::<Result<Vec<_>, _>>()?),
-                            PhantomData),
+                Annotations(Some(Box::new(b.iter().map(|a| a.copy_via(f)).collect::<Result<Vec<_>, _>>()?))),
         })
     }
 }
@@ -1025,35 +1098,35 @@ impl<N: NestedValue<D>, D: Embeddable> Annotations<N, D> {
 /// A possibly-annotated Value, with annotations (themselves
 /// possibly-annotated) in order of appearance.
 #[derive(Clone)]
-pub struct AnnotatedValue<N: NestedValue<D>, D: Embeddable>(pub Annotations<N, D>, pub Value<N, D>);
+pub struct AnnotatedValue<N: NestedValue>(pub Annotations<N>, pub Value<N>);
 
-impl<N: NestedValue<D>, D: Embeddable> AnnotatedValue<N, D> {
-    fn new(anns: Annotations<N, D>, value: Value<N, D>) -> Self {
+impl<N: NestedValue> AnnotatedValue<N> {
+    fn new(anns: Annotations<N>, value: Value<N>) -> Self {
         AnnotatedValue(anns, value)
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> PartialEq for AnnotatedValue<N, D> {
+impl<N: NestedValue> PartialEq for AnnotatedValue<N> {
     fn eq(&self, other: &Self) -> bool {
         self.1.eq(&other.1)
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> Eq for AnnotatedValue<N, D> {}
+impl<N: NestedValue> Eq for AnnotatedValue<N> {}
 
-impl<N: NestedValue<D>, D: Embeddable> Hash for AnnotatedValue<N, D> {
+impl<N: NestedValue> Hash for AnnotatedValue<N> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.1.hash(state);
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> PartialOrd for AnnotatedValue<N, D> {
+impl<N: NestedValue> PartialOrd for AnnotatedValue<N> {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
     }
 }
 
-impl<N: NestedValue<D>, D: Embeddable> Ord for AnnotatedValue<N, D> {
+impl<N: NestedValue> Ord for AnnotatedValue<N> {
     fn cmp(&self, other: &Self) -> Ordering {
         self.1.cmp(&other.1)
     }
@@ -1062,37 +1135,39 @@ impl<N: NestedValue<D>, D: Embeddable> Ord for AnnotatedValue<N, D> {
 //---------------------------------------------------------------------------
 
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct PlainValue<D: Embeddable>(AnnotatedValue<PlainValue<D>, D>);
+pub struct PlainValue<D: Embeddable>(AnnotatedValue<PlainValue<D>>);
 
 impl<D: Embeddable> PlainValue<D> {
-    pub fn annotations_mut(&mut self) -> &mut Annotations<Self, D> {
+    pub fn annotations_mut(&mut self) -> &mut Annotations<Self> {
         &mut (self.0).0
     }
 
-    pub fn value_mut(&mut self) -> &mut Value<Self, D> {
+    pub fn value_mut(&mut self) -> &mut Value<Self> {
         &mut (self.0).1
     }
 }
 
-impl<D: Embeddable> NestedValue<D> for PlainValue<D> {
-    fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self {
+impl<D: Embeddable> NestedValue for PlainValue<D> {
+    type Embedded = D;
+
+    fn wrap(anns: Annotations<Self>, v: Value<Self>) -> Self {
         PlainValue(AnnotatedValue::new(anns, v))
     }
 
-    fn annotations(&self) -> &Annotations<Self, D> {
+    fn annotations(&self) -> &Annotations<Self> {
         &(self.0).0
     }
 
-    fn value(&self) -> &Value<Self, D> {
+    fn value(&self) -> &Value<Self> {
         &(self.0).1
     }
 
-    fn pieces(self) -> (Annotations<Self, D>, Value<Self, D>) {
+    fn pieces(self) -> (Annotations<Self>, Value<Self>) {
         let AnnotatedValue(anns, v) = self.0;
         (anns, v)
     }
 
-    fn value_owned(self) -> Value<Self, D> {
+    fn value_owned(self) -> Value<Self> {
         (self.0).1
     }
 }
@@ -1108,29 +1183,31 @@ impl<D: Embeddable> Debug for PlainValue<D> {
 use std::rc::Rc;
 
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct RcValue<D: Embeddable>(Rc<AnnotatedValue<RcValue<D>, D>>);
+pub struct RcValue<D: Embeddable>(Rc<AnnotatedValue<RcValue<D>>>);
 
-impl<D: Embeddable> NestedValue<D> for RcValue<D> {
-    fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self {
+impl<D: Embeddable> NestedValue for RcValue<D> {
+    type Embedded = D;
+
+    fn wrap(anns: Annotations<Self>, v: Value<Self>) -> Self {
         RcValue(Rc::new(AnnotatedValue::new(anns, v)))
     }
 
-    fn annotations(&self) -> &Annotations<Self, D> {
+    fn annotations(&self) -> &Annotations<Self> {
         &(self.0).0
     }
 
-    fn value(&self) -> &Value<Self, D> {
+    fn value(&self) -> &Value<Self> {
         &(self.0).1
     }
 
-    fn pieces(self) -> (Annotations<Self, D>, Value<Self, D>) {
+    fn pieces(self) -> (Annotations<Self>, Value<Self>) {
         match Rc::try_unwrap(self.0) {
             Ok(AnnotatedValue(anns, v)) => (anns, v),
             Err(r) => (r.0.clone(), r.1.clone()),
         }
     }
 
-    fn value_owned(self) -> Value<Self, D> {
+    fn value_owned(self) -> Value<Self> {
         Rc::try_unwrap(self.0).unwrap_or_else(|_| panic!("value_owned on RcValue with refcount greater than one")).1
     }
 }
@@ -1144,29 +1221,31 @@ impl<D: Embeddable> Debug for RcValue<D> {
 //---------------------------------------------------------------------------
 
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct ArcValue<D: Embeddable>(Arc<AnnotatedValue<ArcValue<D>, D>>);
+pub struct ArcValue<D: Embeddable>(Arc<AnnotatedValue<ArcValue<D>>>);
 
-impl<D: Embeddable> NestedValue<D> for ArcValue<D> {
-    fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self {
+impl<D: Embeddable> NestedValue for ArcValue<D> {
+    type Embedded = D;
+
+    fn wrap(anns: Annotations<Self>, v: Value<Self>) -> Self {
         ArcValue(Arc::new(AnnotatedValue::new(anns, v)))
     }
 
-    fn annotations(&self) -> &Annotations<Self, D> {
+    fn annotations(&self) -> &Annotations<Self> {
         &(self.0).0
     }
 
-    fn value(&self) -> &Value<Self, D> {
+    fn value(&self) -> &Value<Self> {
         &(self.0).1
     }
 
-    fn pieces(self) -> (Annotations<Self, D>, Value<Self, D>) {
+    fn pieces(self) -> (Annotations<Self>, Value<Self>) {
         match Arc::try_unwrap(self.0) {
             Ok(AnnotatedValue(anns, v)) => (anns, v),
             Err(r) => (r.0.clone(), r.1.clone()),
         }
     }
 
-    fn value_owned(self) -> Value<Self, D> {
+    fn value_owned(self) -> Value<Self> {
         Arc::try_unwrap(self.0).unwrap_or_else(|_| panic!("value_owned on ArcValue with refcount greater than one")).1
     }
 }
@@ -1180,32 +1259,34 @@ impl<D: Embeddable> Debug for ArcValue<D> {
 //---------------------------------------------------------------------------
 
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct IOValue(Arc<AnnotatedValue<IOValue, IOValue>>);
-pub type UnwrappedIOValue = Value<IOValue, IOValue>;
+pub struct IOValue(Arc<AnnotatedValue<IOValue>>);
+pub type UnwrappedIOValue = Value<IOValue>;
 
 impl Domain for IOValue {}
 
-impl NestedValue<IOValue> for IOValue {
-    fn wrap(anns: Annotations<Self, IOValue>, v: Value<Self, IOValue>) -> Self {
+impl NestedValue for IOValue {
+    type Embedded = Self;
+
+    fn wrap(anns: Annotations<Self>, v: Value<Self>) -> Self {
         IOValue(Arc::new(AnnotatedValue::new(anns, v)))
     }
 
-    fn annotations(&self) -> &Annotations<Self, IOValue> {
+    fn annotations(&self) -> &Annotations<Self> {
         &(self.0).0
     }
 
-    fn value(&self) -> &Value<Self, IOValue> {
+    fn value(&self) -> &Value<Self> {
         &(self.0).1
     }
 
-    fn pieces(self) -> (Annotations<Self, IOValue>, Value<Self, IOValue>) {
+    fn pieces(self) -> (Annotations<Self>, Value<Self>) {
         match Arc::try_unwrap(self.0) {
             Ok(AnnotatedValue(anns, v)) => (anns, v),
             Err(r) => (r.0.clone(), r.1.clone()),
         }
     }
 
-    fn value_owned(self) -> Value<Self, IOValue> {
+    fn value_owned(self) -> Value<Self> {
         match Arc::try_unwrap(self.0) {
             Ok(AnnotatedValue(_anns, v)) => v,
             Err(r) => r.1.clone(),
@@ -1234,7 +1315,7 @@ impl<'de> serde::Deserialize<'de> for IOValue {
 //---------------------------------------------------------------------------
 
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct DummyValue<D: Embeddable>(AnnotatedValue<DummyValue<D>, D>);
+pub struct DummyValue<D: Embeddable>(AnnotatedValue<DummyValue<D>>);
 
 impl<D: Embeddable> Debug for DummyValue<D> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -1242,31 +1323,32 @@ impl<D: Embeddable> Debug for DummyValue<D> {
     }
 }
 
-
 impl<D: Embeddable> DummyValue<D> {
     pub fn new() -> Self {
         DummyValue(AnnotatedValue::new(Annotations::empty(), Value::Boolean(false)))
     }
 }
 
-impl<D: Embeddable> NestedValue<D> for DummyValue<D> {
-    fn wrap(_anns: Annotations<Self, D>, _v: Value<Self, D>) -> Self {
+impl<D: Embeddable> NestedValue for DummyValue<D> {
+    type Embedded = D;
+
+    fn wrap(_anns: Annotations<Self>, _v: Value<Self>) -> Self {
         DummyValue::new()
     }
 
-    fn annotations(&self) -> &Annotations<Self, D> {
+    fn annotations(&self) -> &Annotations<Self> {
         &self.0.0
     }
 
-    fn value(&self) -> &Value<Self, D> {
+    fn value(&self) -> &Value<Self> {
         &self.0.1
     }
 
-    fn pieces(self) -> (Annotations<Self, D>, Value<Self, D>) {
+    fn pieces(self) -> (Annotations<Self>, Value<Self>) {
         (self.0.0, self.0.1)
     }
 
-    fn value_owned(self) -> Value<Self, D> {
+    fn value_owned(self) -> Value<Self> {
         self.0.1
     }
 }
